// SubThread.cpp: Implementation of CSubThread class
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "SubThread.h"
#include "myDebug.h"
#include <process.h>    /* _beginthread, _endthread */


#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Constructor/Destructor
//////////////////////////////////////////////////////////////////////

CSubThread::CSubThread()
{
	iSt = __ST_FIN;
	hWndTarget	= NULL;
	szErrMsg = "";
	ModePrinter_s=true;
	portName = "";
	portSettings="";
	memset(&StarPrinterStatus_m, 0, sizeof(StarPrinterStatus_2));
	ExecuteMode=__EXEC_READ_STATUS;

	m_HndlThread = NULL;
	m_StopThreadReq = true;
}

CSubThread::~CSubThread()
{
	iSt = __ST_STOP;
}

void CSubThread::m_ThreadProc()
{
	const int TIMEOUT = 2000;
	int result;
	void * port;
	int i;

	while(iSt != __ST_FIN){
		if(	m_StopThreadReq ) iSt = __ST_STOP;
		switch (iSt) {
		case __ST_INIT:
			DebugPrint(_T("SubThread was initialized !\n"));
			if( hWndTarget ){
				szErrMsg = "Sub thread is started by main thread. This message is posted from sub thread";
				PostMessage( hWndTarget, __MSG_SUB_THREAD1, __ST_START, NULL );
			}
			iSt = __ST_START;
			break;

		case __ST_START:
			DebugPrint(_T("SubThread is Started !\n"));
			switch(ExecuteMode)
			{
			case __EXEC_READ_STATUS:
				if(ModePrinter_s == true)	// Printer
				{
					iSt = __ST_STOP;
				}
				else	// StarIO
				{
					// Open Port
					for(i=0;i<5;i++)
					{
						port = OpenPort(portName, portSettings, TIMEOUT);
						if(port != NULL)
						{
							break;
						}
						else
						{
							Sleep(50);	// retry
						}
					}

					if(port == NULL)
					{
						szErrMsg = "Error OpenPort for Getting Status";
						PostMessage( hWndTarget, __MSG_SUB_THREAD1, __ST_ENDOPENNG, NULL );
						iSt = __ST_STOP;
					}
					else
					{
						StarPrinterStatus_2	StarPrinterStatus_r;
						memset(&StarPrinterStatus_r, 0, sizeof(StarPrinterStatus_2));
						// Executes getting status from printer by GetParsedStatusEx() function
						if(result = GetParsedStatusEx(port, &StarPrinterStatus_r,2))
						{
							StarPrinterStatus_m=StarPrinterStatus_r;
							szErrMsg = "Finish Get Status from Printer";
							PostMessage( hWndTarget, __MSG_SUB_THREAD1, __ST_ENDOK, NULL );
							iSt = __ST_STOP;
						}
						else
						{
							szErrMsg = "Error Get Status";
							PostMessage( hWndTarget, __MSG_SUB_THREAD1, __ST_ENDNG, NULL );
							iSt = __ST_STOP;
						}
						// Close Port
						ClosePort(port);
					}
				}
				break;
			case __EXEC_RESET_DEVICE:
				port = OpenPort(portName, portSettings, TIMEOUT);
				if(port == NULL)
				{
					szErrMsg = "Error OpenPort for Reset Device";
					PostMessage( hWndTarget, __MSG_SUB_THREAD1, __ST_ENDOPENNG, NULL );
					iSt = __ST_STOP;
				}
				else
				{
					result = ResetDevice(port);
					szErrMsg = "Finish Reset Device";
					PostMessage( hWndTarget, __MSG_SUB_THREAD1, __ST_ENDOK, NULL );
					iSt = __ST_STOP;
					ClosePort(port);
				}
				break;
			case __EXEC_READ_MODEL:
				port = OpenPort(portName, portSettings, TIMEOUT);
				if(port == NULL)
				{
					szErrMsg = "Error OpenPort for Read ModelVersion";
					PostMessage( hWndTarget, __MSG_SUB_THREAD1, __ST_ENDOPENNG, NULL );
					iSt = __ST_STOP;
				}
				else
				{
					if(ReadModel(port) == 0)
					{
						szErrMsg = "Error Getting Model Name";
						PostMessage( hWndTarget, __MSG_SUB_THREAD1, __ST_ENDNG, NULL );
						iSt = __ST_STOP;
					}
					else
					{
						szErrMsg = "Finish Getting Model Name";
						PostMessage( hWndTarget, __MSG_SUB_THREAD1, __ST_ENDOK, NULL );
						iSt = __ST_STOP;
					}
				}
				break;
			}
			break;

		case __ST_RUN0:
			break;

		case __ST_RUN1:
			break;
		
		case __ST_STOP:
			DebugPrint(_T("SubThread was stopped !\n"));
			if( hWndTarget ){
				szErrMsg = "Sub thread is stopped by main thread. This message is posted from sub thread";
				PostMessage( hWndTarget, __MSG_SUB_THREAD1, __ST_STOP, NULL );
			}
			iSt = __ST_FIN;
			break;

		default:
			iSt = __ST_STOP;
			break;
		}
	}
	DebugPrint(_T("SubThread was finished !\n"));
	m_HndlThread = NULL;
	_endthread();
}


BOOL CSubThread::m_CreateThread()
{
	BOOL	blRes = false;
	// make sub thread if not yet
	if(!m_HndlThread){
		m_StopThreadReq = false;
		iSt = __ST_INIT;
		DebugPrint(_T("// Craete SubThread //\n"))
		// make sub thread
		m_HndlThread = _beginthread( m_EntryPoint, NULL, (LPVOID)this );
		if ( m_HndlThread ){
			return true;
		}else{
			return false;
		}
	}else
	{
		if(iSt == __ST_FIN)
		{
			m_HndlThread = NULL;
		}
		else
		{
			DebugPrint(_T("// Sub thread is already running. So you cannnot start any more thread. //"))
			szErrMsg = "Sub thread is already running. So you cannnot start any more thread.";
			PostMessage( hWndTarget, __MSG_SUB_THREAD1, __WARN_01, NULL );
		}
	}
	return false;
}


BOOL CSubThread::StartSubThread()
{
	BOOL	blRes = false;

	blRes = m_CreateThread();
	return blRes;
}


BOOL CSubThread::StopSubThread()
{
	m_StopThreadReq = true;
	return true;
}

int CSubThread::ReadModel(void * port)
{
	unsigned char command[] = {0x1b,0x1e,'a','2',0x1b, 0x23, 0x2A, 0x0a, 0x00};
	UINT32 commandLength = sizeof(command);
	UINT32 offset = 0;
	while(offset < commandLength)
	{
		UINT32 bytesWritten;
		if(WritePort(port, (UCHAR const *)command, commandLength - offset, &bytesWritten) == 0)
		{
			ClosePort(port);
			return 0;
		}
		offset += bytesWritten;
	}

	UINT32 bytesRead;

	char buffer[100];

	char firmwareName[50];
	memset(firmwareName, 0, sizeof(firmwareName));
	READSTATE state = INIT;
	INT32 nsbToRead = 0;
	bool firmWareNotRead = true;
	INT32 firmwareCounter = 0;
	INT32 readCounter = 0;

	if(PortUSBPrinterClass == true)
	{
		if(ReadPort(port, (UCHAR *)buffer, sizeof(buffer) - 1, &bytesRead) == 0)
		{
			ClosePort(port);
			return 0;
		}
	}
	if(ReadPort(port, (UCHAR *)buffer, sizeof(buffer) - 1, &bytesRead) == 0)
	{
		ClosePort(port);
		return 0;
	}

	if(bytesRead == 0)
	{
		ClosePort(port);
		return 0;
	}

	while(firmWareNotRead == true)
	{
		//Reads a single byte at a time
		char byteAnalize = buffer[readCounter];
		if(readCounter > 99)
		{
			ClosePort(port);
			return 0;
		}
		if(state == INIT)
		{
			// If the read byte is 0x1B then it ready to read the firmware version
			if(byteAnalize == 0x1B)
			{
				state = FIRMSTART;
			}
			// If something else is read then the NSB state is read and one of the NSB headers are matched
			else
			{
				state = NSB;
				switch(byteAnalize)
				{
				case 0x0F:
					nsbToRead = 6;
					break;
				case 0x21:
					nsbToRead = 7;
					break;
				case 0x23:
					nsbToRead = 8;
					break;
				case 0x25:
					nsbToRead = 9;
					break;
				case 0x27:
					nsbToRead = 10;
					break;
				case 0x29:
					nsbToRead = 11;
					break;
				case 0x2B:
					nsbToRead = 12;
					break;
				case 0x2D:
					nsbToRead = 13;
					break;
				case 0x2F:
					nsbToRead = 14;
					break;
				default:
					// If none of the headers are matched then the state is returned to INIT
					state = INIT;
					break;
				}
			}
		}
		// If state is NSB then the NSB information is read until its gone.
		else if(state == NSB)
		{
			nsbToRead--;
			if(nsbToRead == 0)
			{
				state = INIT;
			}
		}
		// If the firmStart state is entered then the header for the firmware version is read.
		else if(state == FIRMSTART)
		{
			if(byteAnalize == 0x2C)
			{
				state = FIRM;
			}
		}
		else if(state == FIRM)
		{
			//The firmware version is read.
			if(byteAnalize == 0x0a)
			{
			}
			else if(byteAnalize == 0x00)
			{
				state = INIT;
				firmWareNotRead = false;
			}
			else
			{
				if(firmwareCounter < 49)
				{
					firmwareName[firmwareCounter] = byteAnalize;
					firmwareCounter++;
				}
			}
		}
		readCounter++;
	}
	int count;
	for(count=0;count<firmwareCounter+1;count++){
		ReadFirmwareName[count]=(unsigned short)firmwareName[count];
	}
	ClosePort(port);
	return 1;
}

////////////////////////////////////////////////////////////////
// Entry point of the sub thread 
void __cdecl CSubThread::m_EntryPoint(void *pParam)
{
	UINT uRes = 1;

	CSubThread *pMyClass = (CSubThread *)pParam;
	pMyClass->m_ThreadProc();
	return;
}
